package com.huertian.jinduguanli.controller;

import com.huertian.jinduguanli.common.ApiResponse;
import com.huertian.jinduguanli.common.ErrorCode;
import com.huertian.jinduguanli.dto.LessonTaskRequest;
import com.huertian.jinduguanli.dto.LessonTaskDTO;
import com.huertian.jinduguanli.entity.LessonTask;
import com.huertian.jinduguanli.service.LessonTaskService;
import jakarta.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api/lesson-tasks")
public class LessonTaskController {
    private static final Logger logger = LoggerFactory.getLogger(LessonTaskController.class);
    private final LessonTaskService lessonTaskService;

    @Autowired
    public LessonTaskController(LessonTaskService lessonTaskService) {
        this.lessonTaskService = lessonTaskService;
    }

    @GetMapping
    public ApiResponse<Page<LessonTask>> listTasks(
            @RequestParam(defaultValue = "1") Integer page,
            @RequestParam(defaultValue = "10") Integer size,
            @RequestParam(required = false) Long userId) {
        logger.info("收到课程任务列表查询请求，页码: {}，每页数量: {}，用户ID: {}", page, size, userId);
        try {
            Page<LessonTask> tasks = lessonTaskService.findAll(PageRequest.of(page - 1, size), userId);
            return ApiResponse.success(tasks);
        } catch (Exception e) {
            logger.error("查询课程任务列表失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "获取课程任务列表失败", null);
        }
    }

    @GetMapping("/{id}")
    public ApiResponse<LessonTask> getTask(@PathVariable Long id) {
        logger.info("收到课程任务查询请求，任务ID: {}", id);
        try {
            LessonTask task = lessonTaskService.findById(id);
            return ApiResponse.success(task);
        } catch (Exception e) {
            logger.error("查询课程任务失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "获取课程任务失败", null);
        }
    }

    @GetMapping("/department/{departmentId}")
    public ApiResponse<Page<LessonTaskDTO>> getTasksByDepartment(
            @PathVariable Long departmentId,
            @RequestParam(defaultValue = "1") Integer page,
            @RequestParam(defaultValue = "10") Integer size) {
        logger.info("收到部门课程任务查询请求，部门ID: {}", departmentId);
        try {
            PageRequest pageRequest = PageRequest.of(page - 1, size);
            Page<LessonTaskDTO> tasks = lessonTaskService.findByDepartmentIdAndNormalUser(departmentId, pageRequest);
            return ApiResponse.success(tasks);
        } catch (Exception e) {
            logger.error("查询部门课程任务失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "获取部门课程任务失败", null);
        }
    }

    @PostMapping
    public ApiResponse<LessonTask> createTask(@Valid @RequestBody LessonTaskRequest request) {
        logger.info("收到创建课程任务请求");
        try {
            LessonTask task = lessonTaskService.create(request);
            return ApiResponse.success(task);
        } catch (Exception e) {
            logger.error("创建课程任务失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "创建课程任务失败", null);
        }
    }

    @PutMapping("/{id}")
    public ApiResponse<LessonTask> updateTask(@PathVariable Long id, @Valid @RequestBody LessonTaskRequest request) {
        logger.info("收到更新课程任务请求，任务ID: {}", id);
        try {
            LessonTask task = lessonTaskService.update(id, request);
            return ApiResponse.success(task);
        } catch (Exception e) {
            logger.error("更新课程任务失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "更新课程任务失败", null);
        }
    }

    @DeleteMapping("/{id}")
    public ApiResponse<Void> deleteTask(@PathVariable Long id) {
        logger.info("收到删除课程任务请求，任务ID: {}", id);
        try {
            lessonTaskService.delete(id);
            return ApiResponse.success();
        } catch (Exception e) {
            logger.error("删除课程任务失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "删除课程任务失败", null);
        }
    }
}
