package com.huertian.jinduguanli.controller;

import com.huertian.jinduguanli.common.ApiResponse;
import com.huertian.jinduguanli.common.ErrorCode;
import com.huertian.jinduguanli.dto.CreateUserRequest;
import com.huertian.jinduguanli.dto.LoginResponse;
import com.huertian.jinduguanli.dto.UserLoginRequest;
import com.huertian.jinduguanli.entity.User;
import com.huertian.jinduguanli.security.service.JwtService;
import com.huertian.jinduguanli.security.service.TokenBlacklistService;
import com.huertian.jinduguanli.service.UserService;
import jakarta.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api/users")
public class UserController {
    private static final Logger logger = LoggerFactory.getLogger(UserController.class);

    private final UserService userService;
    private final JwtService jwtService;
    private final TokenBlacklistService tokenBlacklistService;

    @Autowired
    public UserController(
            UserService userService,
            JwtService jwtService,
            TokenBlacklistService tokenBlacklistService) {
        this.userService = userService;
        this.jwtService = jwtService;
        this.tokenBlacklistService = tokenBlacklistService;
    }

    @PostMapping("/login")
    public ApiResponse<LoginResponse> login(@Valid @RequestBody UserLoginRequest request) {
        logger.info("收到登录请求，邮箱: {}", request.getEmail());
        ApiResponse<LoginResponse> response = userService.login(request);
        logger.info("登录响应: {}", response);
        return response;
    }

    @PostMapping("/logout")
    public ApiResponse<String> logout(@RequestHeader("Authorization") String authHeader) {
        if (authHeader != null && authHeader.startsWith("Bearer ")) {
            String token = authHeader.substring(7);
            // 获取token的过期时间
            long expiration = jwtService.extractExpiration(token).getTime();
            // 将token加入黑名单
            tokenBlacklistService.addToBlacklist(token, expiration);
            return ApiResponse.success("登出成功");
        }
        return ApiResponse.success("登出成功");
    }

    @PostMapping
    public ApiResponse<Void> createUser(@Valid @RequestBody CreateUserRequest request) {
        logger.info("收到创建用户请求，用户名: {}", request.getUsername());
        return userService.createUser(request);
    }

    @GetMapping("/current")
    public ApiResponse<User> getCurrentUser(@RequestHeader("Authorization") String authHeader) {
        logger.info("收到获取当前用户信息请求，Authorization: {}", authHeader);
        if (authHeader != null && authHeader.startsWith("Bearer ")) {
            String token = authHeader.substring(7);
            logger.info("解析出的token: {}", token);
            return userService.getCurrentUser(token);
        }
        logger.warn("未提供有效的认证令牌");
        return ApiResponse.error(401, "未提供有效的认证令牌");
    }

    @GetMapping("/list")
    public ApiResponse<?> listUsers(
            @RequestParam(defaultValue = "1") Integer page,
            @RequestParam(defaultValue = "10") Integer limit) {
        logger.info("收到用户列表分页查询请求，页码: {}，每页数量: {}", page, limit);
        return userService.listUsers(page, limit);
    }

    @PostMapping("/disable/{userId}")
    public ApiResponse<String> disableUser(@PathVariable Long userId) {
        logger.info("收到禁用用户请求，用户ID: {}", userId);
        return userService.disableUser(userId);
    }

    @GetMapping("/department/{departmentId}")
    public ApiResponse<List<User>> getUsersByDepartment(@PathVariable Long departmentId) {
        logger.info("收到查询部门用户请求，部门ID: {}", departmentId);
        try {
            List<User> users = userService.findByDepartmentId(departmentId);
            // 清除密码信息
            users.forEach(user -> user.setPassword(null));
            return ApiResponse.success(users);
        } catch (Exception e) {
            logger.error("查询部门用户失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "获取部门用户列表失败", null);
        }
    }

    @GetMapping("/teacher/search")
    public ApiResponse<User> searchTeacher(@RequestParam String email) {
        logger.info("收到查询教师信息请求，邮箱: {}", email);
        try {
            return userService.findTeacherByEmail(email);
        } catch (Exception e) {
            logger.error("查询教师信息失败", e);
            return new ApiResponse<>(ErrorCode.SYSTEM_ERROR, "查询教师信息失败", null);
        }
    }
}
