package com.huertian.jinduguanli.security.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.concurrent.TimeUnit;

@Service
public class TokenBlacklistService {
    private final StringRedisTemplate redisTemplate;
    private static final String BLACKLIST_PREFIX = "token:blacklist:";
    private static final Logger logger = LoggerFactory.getLogger(TokenBlacklistService.class);

    public TokenBlacklistService(StringRedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    /**
     * 将令牌添加到黑名单
     * 
     * @param token      JWT令牌
     * @param expiration 过期时间（秒）
     */
    public void addToBlacklist(String token, long expiration) {
        logger.info("将令牌加入黑名单 - 过期时间: {}", new Date(expiration * 1000));
        try {
            String key = BLACKLIST_PREFIX + token;
            long ttl = expiration - System.currentTimeMillis() / 1000;
            if (ttl > 0) {
                redisTemplate.opsForValue().set(key, "blacklisted", ttl, TimeUnit.SECONDS);
                logger.info("令牌已成功加入黑名单");
            } else {
                logger.warn("令牌已过期，无需加入黑名单");
            }
        } catch (Exception e) {
            logger.error("将令牌加入黑名单时发生错误", e);
            throw new RuntimeException("无法将令牌加入黑名单", e);
        }
    }

    /**
     * 检查令牌是否在黑名单中
     * 
     * @param token JWT令牌
     * @return 如果在黑名单中返回true
     */
    public boolean isBlacklisted(String token) {
        logger.debug("检查令牌是否在黑名单中");
        try {
            String key = BLACKLIST_PREFIX + token;
            Boolean exists = redisTemplate.hasKey(key);
            logger.debug("令牌黑名单检查结果: {}", exists);
            return Boolean.TRUE.equals(exists);
        } catch (Exception e) {
            logger.error("检查令牌黑名单状态时发生错误", e);
            return false;
        }
    }
}
