package com.huertian.jinduguanli.service;

import com.huertian.jinduguanli.dto.LessonTaskDTO;
import com.huertian.jinduguanli.dto.LessonTaskRequest;
import com.huertian.jinduguanli.entity.LessonTask;
import com.huertian.jinduguanli.repository.LessonTaskRepository;
import jakarta.persistence.EntityNotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.BeanUtils;

@Service
public class LessonTaskService {
    private static final Logger logger = LoggerFactory.getLogger(LessonTaskService.class);
    private final LessonTaskRepository lessonTaskRepository;

    public LessonTaskService(LessonTaskRepository lessonTaskRepository) {
        this.lessonTaskRepository = lessonTaskRepository;
    }

    @Cacheable(value = "lessonTasks", key = "#userId != null ? 'user:' + #userId + ':page:' + #pageable.pageNumber : 'all:page:' + #pageable.pageNumber")
    public Page<LessonTask> findAll(Pageable pageable, Long userId) {
        logger.info("查询课程任务 - 页码: {}, 每页数量: {}, 用户ID: {}",
                pageable.getPageNumber(), pageable.getPageSize(), userId);

        Page<LessonTask> result;
        if (userId != null) {
            result = lessonTaskRepository.findByUserId(userId, pageable);
        } else {
            result = lessonTaskRepository.findAll(pageable);
        }

        logger.info("查询到 {} 条课程任务", result.getTotalElements());
        return result;
    }

    @Cacheable(value = "lessonTask", key = "#id")
    public LessonTask findById(Long id) {
        logger.info("根据ID查询课程任务 - 任务ID: {}", id);
        return lessonTaskRepository.findById(id)
                .orElseThrow(() -> {
                    logger.error("未找到ID为 {} 的课程任务", id);
                    return new EntityNotFoundException("未找到ID为 " + id + " 的任务");
                });
    }

    @Transactional
    @CacheEvict(value = { "lessonTasks", "lessonTask" }, allEntries = true)
    public LessonTask create(LessonTaskRequest request) {
        logger.info("开始创建课程任务 - 课程名称: {}, 微课名称: {}, 用户ID: {}",
                request.getCourseName(), request.getMicroLessonName(), request.getUserId());
        validateRequest(request);
        LessonTask task = new LessonTask();
        BeanUtils.copyProperties(request, task);
        task.setProgressStatus(0); // 初始状态：未开始
        LessonTask savedTask = lessonTaskRepository.save(task);
        logger.info("创建课程任务成功 - 任务ID: {}", savedTask.getId());
        return savedTask;
    }

    @Transactional
    @CacheEvict(value = { "lessonTasks", "lessonTask" }, allEntries = true)
    public LessonTask update(Long id, LessonTaskRequest request) {
        logger.info("开始更新课程任务 - 任务ID: {}, 进度状态: {}", id, request.getProgressStatus());
        LessonTask task = findById(id);

        // 只更新非空字段
        if (request.getProgressStatus() != null) {
            task.setProgressStatus(request.getProgressStatus());
            // 根据进度状态更新时间戳
            long currentTime = System.currentTimeMillis() / 1000;
            switch (request.getProgressStatus()) {
                case 0: // 未开始
                    task.setScriptCreateTime(null);
                    task.setScriptReviewTime(null);
                    task.setScriptConfirmTime(null);
                    task.setVideoCreateTime(null);
                    task.setVideoConfirmTime(null);
                    task.setFinishTime(null);
                    break;
                case 1: // 脚本制作
                    task.setScriptCreateTime(currentTime);
                    task.setScriptReviewTime(null);
                    task.setScriptConfirmTime(null);
                    task.setVideoCreateTime(null);
                    task.setVideoConfirmTime(null);
                    task.setFinishTime(null);
                    break;
                case 2: // 脚本审核
                    task.setScriptReviewTime(currentTime);
                    task.setScriptConfirmTime(null);
                    task.setVideoCreateTime(null);
                    task.setVideoConfirmTime(null);
                    task.setFinishTime(null);
                    break;
                case 3: // 脚本确认
                    task.setScriptConfirmTime(currentTime);
                    task.setVideoCreateTime(null);
                    task.setVideoConfirmTime(null);
                    task.setFinishTime(null);
                    break;
                case 4: // 视频拍摄与制作
                    task.setVideoCreateTime(currentTime);
                    task.setVideoConfirmTime(null);
                    task.setFinishTime(null);
                    break;
                case 5: // 视频确认
                    task.setVideoConfirmTime(currentTime);
                    task.setFinishTime(currentTime);
                    break;
            }
        }
        if (request.getAdvise() != null) {
            task.setAdvise(request.getAdvise());
        }
        if (request.getCourseName() != null) {
            task.setCourseName(request.getCourseName());
        }
        if (request.getMicroLessonName() != null) {
            task.setMicroLessonName(request.getMicroLessonName());
        }
        if (request.getUserId() != null) {
            task.setUserId(request.getUserId());
        }

        task.setUpdatedAt(System.currentTimeMillis() / 1000);
        LessonTask updatedTask = lessonTaskRepository.save(task);
        logger.info("更新课程任务成功 - 任务ID: {}", updatedTask.getId());
        return updatedTask;
    }

    @Transactional
    @CacheEvict(value = { "lessonTasks", "lessonTask" }, allEntries = true)
    public void delete(Long id) {
        logger.info("开始删除课程任务 - 任务ID: {}", id);
        if (!lessonTaskRepository.existsById(id)) {
            logger.error("未找到ID为 {} 的课程任务", id);
            throw new EntityNotFoundException("未找到ID为 " + id + " 的任务");
        }
        lessonTaskRepository.deleteById(id);
        logger.info("删除课程任务成功 - 任务ID: {}", id);
    }

    /**
     * 根据部门ID查询正常状态用户的课程任务列表（分页），包含用户名
     *
     * @param departmentId 部门ID
     * @param pageable     分页参数
     * @return 课程任务分页数据
     */
    public Page<LessonTaskDTO> findByDepartmentIdAndNormalUser(Long departmentId, Pageable pageable) {
        logger.info("查询部门正常用户的课程任务，部门ID: {}", departmentId);
        Page<java.util.Map<String, Object>> result = lessonTaskRepository
                .findByDepartmentIdAndNormalUserWithUsername(departmentId, pageable);

        return result.map(map -> {
            LessonTask lessonTask = (LessonTask) map.get("lessonTask");
            String username = (String) map.get("username");
            return LessonTaskDTO.fromEntity(lessonTask, username);
        });
    }

    /**
     * 根据部门ID和状态查询课程任务列表（分页）
     *
     * @param departmentId 部门ID
     * @param status       任务状态（可选）：0-未开始, 1-脚本制作, 2-脚本审核, 3-脚本确认, 4-视频拍摄与制作, 5-视频确认
     * @param pageable     分页参数
     * @return 课程任务分页数据
     */
    public Page<LessonTask> findByDepartmentIdAndStatus(Long departmentId, Integer status, Pageable pageable) {
        logger.info("查询部门课程任务，部门ID: {}, 状态: {}", departmentId, status);
        return lessonTaskRepository.findByDepartmentIdAndStatus(departmentId, status, pageable);
    }

    /**
     * 根据部门ID和用户状态查询课程任务列表（分页）
     *
     * @param departmentId 部门ID
     * @param userStatus   用户状态（可选）：1-正常, 0-禁用
     * @param pageable     分页参数
     * @return 课程任务分页数据
     */
    public Page<LessonTask> findByDepartmentIdAndUserStatus(Long departmentId, Integer userStatus, Pageable pageable) {
        logger.info("查询部门课程任务，部门ID: {}, 用户状态: {}", departmentId, userStatus);
        return lessonTaskRepository.findByDepartmentIdAndUserStatus(departmentId, userStatus, pageable);
    }

    private void validateRequest(LessonTaskRequest request) {
        if (request.getCourseName() == null || request.getCourseName().trim().isEmpty()) {
            throw new IllegalArgumentException("课程名称不能为空");
        }
        if (request.getMicroLessonName() == null || request.getMicroLessonName().trim().isEmpty()) {
            throw new IllegalArgumentException("微课名称不能为空");
        }
        if (request.getUserId() == null) {
            throw new IllegalArgumentException("用户ID不能为空");
        }
    }
}
