package com.huertian.jinduguanli.service;

import com.huertian.jinduguanli.common.ApiResponse;
import com.huertian.jinduguanli.common.ErrorCode;
import com.huertian.jinduguanli.dto.CreateUserRequest;
import com.huertian.jinduguanli.dto.LoginResponse;
import com.huertian.jinduguanli.dto.UserLoginRequest;
import com.huertian.jinduguanli.dto.UserPageResponse;
import com.huertian.jinduguanli.entity.User;
import com.huertian.jinduguanli.repository.UserRepository;
import com.huertian.jinduguanli.security.service.JwtService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class UserService implements UserDetailsService {
    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder;
    private final JwtService jwtService;
    private static final Logger logger = LoggerFactory.getLogger(UserService.class);

    @Autowired
    public UserService(JwtService jwtService, PasswordEncoder passwordEncoder, UserRepository userRepository) {
        this.jwtService = jwtService;
        this.passwordEncoder = passwordEncoder;
        this.userRepository = userRepository;
    }

    @Override
    public UserDetails loadUserByUsername(String email) throws UsernameNotFoundException {
        Optional<User> user = userRepository.findByEmail(email);
        if (user.isEmpty()) {
            throw new UsernameNotFoundException("User not found with email: " + email);
        }
        return user.get();
    }

    public ApiResponse<LoginResponse> login(UserLoginRequest request) {
        Optional<User> userOpt = userRepository.findByEmail(request.getEmail());
        if (userOpt.isEmpty() || userOpt.get().getStatus() != 1) {
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND_OR_DISABLED, "用户不存在或已禁用", null);
        }

        User user = userOpt.get();
        if (!passwordEncoder.matches(request.getPassword(), user.getPassword())) {
            return new ApiResponse<>(ErrorCode.PASSWORD_INCORRECT, "密码错误", null);
        }

        String token = jwtService.generateToken(user);
        return ApiResponse.success(new LoginResponse(token));
    }

    public ApiResponse<Void> createUser(CreateUserRequest request) {
        // 检查邮箱是否已存在
        if (userRepository.findByEmail(request.getEmail()).isPresent()) {
            return new ApiResponse<>(ErrorCode.EMAIL_EXISTS, "邮箱已存在", null);
        }

        User user = new User();
        user.setEmail(request.getEmail());
        user.setUsername(request.getUsername());
        user.setPassword(passwordEncoder.encode(request.getPassword()));
        user.setDepartmentId(request.getDepartmentId());
        user.setRoles(request.getRoles());
        user.setJobs(request.getJobs());
        user.setCreatorId(request.getCreatorId());
        user.setStatus(1);
        // 时间戳由 @PrePersist 处理，这里不需要手动设置
        
        userRepository.save(user);

        return ApiResponse.success();
    }

    public ApiResponse<User> getCurrentUser(String token) {
        String email = jwtService.extractUsername(token);
        logger.info("从token中提取的邮箱: {}", email);
        Optional<User> userOpt = userRepository.findByEmail(email);
        if (userOpt.isEmpty()) {
            logger.warn("用户不存在: {}", email);
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND, "用户不存在", null);
        }

        User user = userOpt.get();
        logger.info("找到用户: {}", user.getUsername());
        user.setPassword(null);
        return ApiResponse.success(user);
    }

    public ApiResponse<String> disableUser(Long userId) {
        Optional<User> userOpt = userRepository.findById(userId);

        if (userOpt.isEmpty()) {
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND, "用户不存在", null);
        }

        User user = userOpt.get();
        user.setStatus(0);
        userRepository.save(user);

        return ApiResponse.success("用户已禁用");
    }

    public ApiResponse<UserPageResponse> listUsers(Integer currentPage, Integer pageSize) {
        // 参数校验
        if (currentPage < 1) {
            currentPage = 1;
        }
        if (pageSize < 1) {
            pageSize = 10;
        }

        // 查询数据
        Page<User> userPage = userRepository.findAll(PageRequest.of(currentPage - 1, pageSize));

        // 处理返回数据
        UserPageResponse response = new UserPageResponse();
        response.setTotal(userPage.getTotalElements());
        response.setCurrentPage(currentPage);
        response.setPageSize(pageSize);

        List<User> users = userPage.getContent();
        users.forEach(user -> user.setPassword(null));
        response.setList(users);

        return ApiResponse.success(response);
    }

    public User register(User user) {
        // 检查邮箱是否已存在
        if (userRepository.findByEmail(user.getEmail()).isPresent()) {
            throw new RuntimeException("邮箱已存在");
        }

        // 检查用户名是否已存在
        if (userRepository.findByUsername(user.getUsername()).isPresent()) {
            throw new RuntimeException("用户名已存在");
        }

        // 设置默认值
        user.setDepartmentId(0L);
        user.setRoles(0);
        user.setJobs(0);
        user.setCreatorId(0L);

        // 加密密码
        user.setPassword(passwordEncoder.encode(user.getPassword()));
        user.setStatus(1);
        user.setCreatedAt(System.currentTimeMillis() / 1000);
        user.setUpdatedAt(System.currentTimeMillis() / 1000);

        // 保存用户
        User savedUser = userRepository.save(user);

        // 清除密码后返回
        savedUser.setPassword(null);
        return savedUser;
    }

    public List<User> findByDepartmentId(Long departmentId) {
        logger.info("查询部门用户列表，部门ID: {}", departmentId);
        return userRepository.findByDepartmentIdAndNormalStatus(departmentId);
    }

    public ApiResponse<User> findTeacherByEmail(String email) {
        if (email == null || email.trim().isEmpty()) {
            return new ApiResponse<>(ErrorCode.INVALID_PARAM, "邮箱不能为空", null);
        }

        User user = userRepository.findByEmail(email).orElse(null);
        if (user == null) {
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND, "用户不存在", null);
        }

        // 验证用户是否为教师角色且为课程制作教师岗位
        if (user.getRoles() != 1) {
            return new ApiResponse<>(ErrorCode.INVALID_PARAM, "非教师用户", null);
        }

        if (user.getJobs() != 1) {
            return new ApiResponse<>(ErrorCode.INVALID_PARAM, "非课程制作教师", null);
        }

        if (user.getStatus() != 1) {
            return new ApiResponse<>(ErrorCode.INVALID_PARAM, "用户状态异常", null);
        }

        // 清除敏感信息
        user.setPassword(null);
        
        return ApiResponse.success(user);
    }
}
