package com.huertian.jinduguanli.utils;

import com.huertian.jinduguanli.entity.User;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.io.Decoders;
import io.jsonwebtoken.security.Keys;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.security.Key;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

@Component
public class JwtUtil {
    private static final Logger logger = LoggerFactory.getLogger(JwtUtil.class);

    @Value("${jwt.secret}")
    private String SECRET_KEY;

    @Value("${jwt.expiration}")
    private long jwtExpiration;

    public String extractUsername(String token) {
        logger.debug("开始从token中提取用户名");
        try {
            String username = extractClaim(token, Claims::getSubject);
            logger.debug("提取的用户名: {}", username);
            return username;
        } catch (ExpiredJwtException e) {
            logger.error("token已过期", e);
            throw e;
        } catch (Exception e) {
            logger.error("从token中提取用户名时发生错误", e);
            throw e;
        }
    }

    public Date extractExpiration(String token) {
        return extractClaim(token, Claims::getExpiration);
    }

    public <T> T extractClaim(String token, Function<Claims, T> claimsResolver) {
        final Claims claims = extractAllClaims(token);
        return claimsResolver.apply(claims);
    }

    private Claims extractAllClaims(String token) {
        logger.debug("开始从token中提取所有声明");
        try {
            Claims claims = Jwts
                    .parserBuilder()
                    .setSigningKey(getSignKey())
                    .build()
                    .parseClaimsJws(token)
                    .getBody();
            logger.debug("声明提取成功");
            return claims;
        } catch (Exception e) {
            logger.error("从token中提取声明时发生错误", e);
            throw e;
        }
    }

    private Boolean isTokenExpired(String token) {
        try {
            Date expiration = extractExpiration(token);
            boolean isExpired = expiration.before(new Date());
            logger.debug("token过期检查 - 过期时间: {}, 是否过期: {}", expiration, isExpired);
            return isExpired;
        } catch (Exception e) {
            logger.error("检查token过期时发生错误", e);
            throw e;
        }
    }

    public String generateToken(String userName) {
        logger.debug("开始为用户创建token: {}", userName);
        try {
            Map<String, Object> claims = new HashMap<>();
            String token = Jwts.builder()
                    .setClaims(claims)
                    .setSubject(userName)
                    .setIssuedAt(new Date(System.currentTimeMillis() / 1000 * 1000))
                    .setExpiration(new Date((System.currentTimeMillis() / 1000 + jwtExpiration / 1000) * 1000))
                    .signWith(getSignKey(), SignatureAlgorithm.HS384)
                    .compact();
            logger.debug("token创建成功");
            return token;
        } catch (Exception e) {
            logger.error("创建token时发生错误", e);
            throw e;
        }
    }

    public Boolean validateToken(String token, User user) {
        logger.debug("开始验证用户token: {}", user.getUsername());
        try {
            final String username = extractUsername(token);
            boolean isValid = username.equals(user.getUsername()) && !isTokenExpired(token);
            logger.debug("token验证结果 - 用户名匹配: {}, 未过期: {}",
                    username.equals(user.getUsername()), !isTokenExpired(token));
            return isValid;
        } catch (Exception e) {
            logger.error("验证token时发生错误", e);
            return false;
        }
    }

    private Key getSignKey() {
        byte[] keyBytes = Decoders.BASE64.decode(SECRET_KEY);
        return Keys.hmacShaKeyFor(keyBytes);
    }
}
